# mysteriousMystery.py

# Added some more feature to the final version of rpg_gui.py from Chapter 7:
#   - bigger map
#   - 'A' button dialogue
#   - Background music (most WAV files found on freesound.org, original file names kept so traceable. Most have been shortened)
#   - To facilitate cross platform sound this imports the simpleAudio module produced by David Kosbie (kosbie.net)
#     (This has only been tested on Macs, Windows and Raspberry Pis)
#   - Noisy text (turn off by setting teletype_sfx variable to False)
#   - Program responds to stage directions
#   - Acts end when 3 checkpoints are passed.
#     (It is important that the story writer understands to break the story up like this.
#      Checkpoints must always appear before the final line of dialogue.)
#   - Code is arranged so the story with stage directions is right at the top just after preferences.

# NOTES FOR AUTHORS:
#   - Checkpoints must always appear before final dialogue
#   - To ensure players read most of the story, 3 checkpoints need to be passed.
#     CHECKPOINT must always appear before the final line of dialogue.
#   - Currently, only one character is allowed to ENTER per scene.
#   - Stage Directions allowed: ADD, BACKDROP, CHECKPOINT, ENTER, EXIT, MUSIC, TELETYPE
#     ADD and ENTER must include coordinates with 3 characters each coordinate
#     e.g. "ENTER:100,400,Mike" or "ADD:-10,230,mobile" or "ENTER:080,400,Tina"

# Preferences variables:
background_music = True
teletype_sfx = True

####################################################################
# Create stories for each scene:
acts = (
### Act 0
    {
        "Crime Scene": ("ENTER:350,400,Julian",
                       "MUSIC:Rain",
                       "TELETYPE:Off",
                       "Somewhere in London, late at night ... [A]",
                       "TELETYPE:On",
                       "EXIT:Julian",
                       "BACKDROP:crime_scene",
                       "NORAH: What are you doing? Wh.. what is that on your clothes? [A]",
                       "????: I.. I... I don't know! [A]",
                       "TELETYPE:Off",
                       "[To continue the game you should choose where you would like to go by using the arrow keys. Don't forget to look for items and collect them in your bag.] [A]",
                       "TELETYPE:On",
                       "NORAH: Stop. Who are you? You cannot leave. [A]",
                       "CHECKPOINT",
                       "????: I am sorry, I cannot wait. There is somewhere I have to be. [A]",
                       "BACKDROP:crime_scene_dark",
                       "NORAH: Who turned the lights off?"),
        "Detention Centre": ("MUSIC:Cell",
                            "The Detention Centre ..."),
        "Cambridge Street": ("MUSIC:Street",
                            "Cambridge Street ... [A]",
                            "ENTER:100,400,Mike",
                            "Inspector METHODICAL: Morning All! Inspector Mike Methodical of the Met, at your service ... [A]",
                            "CHECKPOINT",
                            "Inspector METHODICAL: I hate wearing ties but my job requires it!"),
        "Hotel Reception": ("MUSIC:Reception",
                           "Hotel Reception ..."),
        "Hotel Manager's Office":("MUSIC:Managers_office",
                                  "CHECKPOINT", 
                                  "The Hotel Manager's Office ..."),
        "Behind the Hotel": ("MUSIC:Park",
                            "Behind the hotel ..."),
        "Julian's Office": ("MUSIC:Detectives_office",
                           "Julian Jones' Office ...")
    },


### Act 1
    {
        "Crime Scene": ("MUSIC:Stop",
                       "The Crime Scene ... [A]",
                       "Someone should really turn on the lights ..."),
        "Detention Centre": ("ENTER:100,400,Tina",
                            "MUSIC:Cell",
                            "The Detention Centre ... [A]", 
                            "TINA: I know it looks hopeless but I did not do it.\nPlease Mr Jones you have to help me. [A]",
                            "JULIAN: I am inclined to believe you Captain Tenable but you were caught at the scene on camera, with a coat soaked in blood. The police found your fingerprints on the car boot where the victim's body was found. [A]",
                            "JULIAN: It looks like the lab report is also going to confirm that the blood on your coat is the same as the victim's. [A]",
                            "JULIAN: And you tell me you remember nothing of the events yesterday - until you awoke in the Cambridge Street Hospital? [A]",
                            "TINA: I am afraid that is true ... [A]",
                            "TINA: I am so sorry ... [A]",
                            "JULIAN: But how can you know you didn't do it then Captain Tenable? [A]",
                            "CHECKPOINT", 
                            "TINA: ... "),
        "Cambridge Street": ("MUSIC:Street",
                            "Cambridge Street ..."),
        "Hotel Reception": ("ENTER:260,400,Arnold",
                           "MUSIC:Reception",
                           "Hotel Reception ... [A]",
                           "ARNOLD: Hello Mr Jones. So you are on this case too. [A]",
                           "JULIAN: I am indeed Officer. Have you found anything out that might interest me? Did any of the guests report seeing the victim around the hotel on the day she died? [A]",
                           "ARNOLD: Indeed they did Mr Jones. The victim's name was Ms Eva Castle ... [A]", 
                           "... and the bellboy tells me she was in and out all day with Captain Tina Tenable ... [A]",
                           "CHECKPOINT", 
                           "... best of friends, they seemed, he tells me."),
        "Hotel Manager's Office": ("ENTER:100,400,Produkt",
                                  "MUSIC:Managers_office",
                                  "The Hotel Manager's Office ... [A]", 
                                  "JULIAN: Thank you for seeing me Herr Produkt. [A]", 
                                  "PRODUKT: I am a very busy man Mr Jones. Ask your questions quickly, I have a lot to do. [A]",
                                  "JONES: Um ... [A]",
                                  "EXIT:Produkt",
                                  "JONES: Herr Produkt ...  [A]",
                                  "CHECKPOINT",
                                  "JONES: Where has Herr Produkt gone? He sure was fast!"),
        "Behind the Hotel": ("MUSIC:Park",
                            "Behind the hotel ..."),
        "Julian's Office": ("MUSIC:Detectives_office", "Julian Jones' Office ...")
    },


### Act 2 - mobile to be found in Hotel Manager's Office
    {
        "Crime Scene": ("MUSIC:Stop",
                       "BACKDROP:crime_scene2",
                       "The Crime Scene ..."),
        "Detention Centre": ("MUSIC:Cell",
                            "The Detention Centre ..."),
        "Cambridge Street": ("MUSIC:Street",
                            "Cambridge Street ..."),
        "Hotel Reception": ("ENTER:100,400,Arnold",
                           "MUSIC:Reception",
                           "Hotel Reception ... [A]",
                           "ARNOLD: Mr Jones, the receptionist found Ms Castle's mobile left on his desk ... [A]", 
                           "He said he gave it to the Manager today because Herr Produkt was away visiting another one of his hotels yesterday. [A]",
                           "JULIAN: You have been very helpful Arnold. Have you collected the phone as evidence? [A]",
                           "CHECKPOINT",
                           "ARNOLD: Um, no. Do you think I should have? "),
        "Hotel Manager's Office": ("ADD:-10,230,mobile",
                                  "MUSIC:Managers_office",
                                  "CHECKPOINT",
                                  "The Hotel Manager's Office ..."),
        "Behind the Hotel": ("MUSIC:Park",
                            "Behind the hotel ..."),
        "Julian's Office": ("ENTER:215,525,Demi",
                           "MUSIC:Detectives_office",
                           "Julian Jones' Office ... [A]",
                           "MS. DEMURE : My name is Demi Demure, Mr Jones. I have an important investigation for you. [A]",
                           "JULIAN: Nice to meet you. What is it you would like me to invesigate? [A]",
                           "MS. DEMURE: There has been a kidnapping. Frederika has been taken. I must get her back alive Mr Jones. [A]",
                           "MS. DEMURE: sniff, sniff ... [A]",
                           "JULIAN: How is Frederika related to you Madam? [A]", 
                           "MS. DEMURE: She is my Koi Carp, Mr Jones. I must get her back, she means the world to me. [A]",
                           "JULIAN: I am afraid I am booked up with important cases for the next three years madam ... [A]",
                           "JULIAN: I suggest you try another investigator such as Mike Methodical, I can give you his number? ... [A]",
                           "EXIT:Demi",
                           "CHECKPOINT",
                           "JULIAN: Wow she must really love that fish!")
    },


### Act 3 - photo to be found behind the hotel
    {
        "Crime Scene": ("MUSIC:Stop",
                       "The Crime Scene ..."),
        "Detention Centre": ("MUSIC:Cell",
                            "The Detention Centre ..."),
        "Cambridge Street": ("ENTER:100,400,Mike",
                            "MUSIC:Street",
                            "Cambridge Street ... [A]",
                            "INSPECTOR METHODICAL: So you are investigating the Car Boot Murder are you Mr Wright? [A]",
                            "INSPECTOR METHODICAL: Doesn't need a lot of investigating. Captain Tenable did it - it's obvious! [A]",
                            "JULIAN: I am inclined to believe Captain Tenable is innocent Inspector. [A]",
                            "INSPECTOR METHODICAL: You always were a sucker for a pretty face Jones. [A]",
                            "JULIAN: Captain Tenable is a war hero. She has risked her life to save others on several occasions. [A]",
                            "ENTER:050,450,Mike_head",
                            "INSPECTOR METHODICAL: Um, ugh ...\nWell you cannot argue with the evidence ... [A]", 
                            "INSPECTOR METHODICAL: Um, urggh ...\nAn investigator like you should know that ... [A]",
                            "ENTER:100,400,Mike",
                            "CHECKPOINT",
                            "INSPECTOR METHODICAL: See you around Mr Jones, I have work to do."),
        "Hotel Reception": ("MUSIC:Reception",
                           "Hotel Reception ..."),
        "Hotel Manager's Office": ("ENTER:100,400,Produkt",
                                  "MUSIC:Managers_office",
                                  "The Hotel Manager's Office ... [A]",
                                  "CHECKPOINT",
                                  "PRODUKT: Where is that blasted mobile phone - I can't find it anywhere."),
        "Behind the Hotel": ("ADD:120,280,photo",
                            "ENTER:130,430,Norah",
                            "MUSIC:Park",
                            "Behind the hotel ... [A]",
                            "JULIAN: Hello Norah, wasn't it you who took the photo? [A]",
                            "NORAH: It was Julian. I am afraid it was not one of my best. It was very dark and I was thoroughly shocked. [A]",
                            "NORAH: I came back today to see if I could be any help but I am afraid it has simply made me feel a bit odd ... [A]",
                            "I just popped out here for some fresh air. [A]",
                            "JULIAN: Norah, is that a copy of the photo you gave the police? [A]", 
                            "NORAH: Yes Julian. I wanted to see if I could make sense of it at the crime scene but I couldn't ... [A]",
                            "... Everything seems to been cleaned up. No blood anywhere, not even on the car. [A]",
                            "JULIAN: That is odd ... [A]", 
                            " ... I was assured that nothing has been touched ... [A]", 
                            " ... Would you mind if I had a look at your photo? [A]",
                            "CHECKPOINT",
                            "NORAH: Help yourself Julian. You can keep it. I can always print out another."),
        "Julian's Office": ("MUSIC:Detectives_office", "Julian Jones' Office ...")
    },


### Act 4 - Report to be found in Julian's Office
    {
        "Crime Scene": ("MUSIC:Rain",
                       "The Crime Scene ... [A]",
                       "CHECKPOINT",
                       "JULIAN: I have never seen anyone return to the Crime Scene."),
        "Detention Centre": ("ENTER:100,400,Tina",
                            "MUSIC:Cell",
                            "The Detention Centre ... [A]",
                            "TINA: Please Mr Jones be straight with me – is it hopeless? [A]",
                            "JULIAN: I am getting very near the end of my investigation Captain Tenable. [A]",
                            "JULIAN: All I can say is I am sure I am going to get to the complete truth of what happened to poor Ms Castle very soon! [A]",
                            "JULIAN: Never give up hope Captain. "),
        "Cambridge Street": ("ENTER:100,400,Mike",
                            "MUSIC:Street",
                            "Cambridge Street ... [A]",
                            "INSPECTOR METHODICAL: Sorry Mr Jones, I am very busy... [A]",
                            "... I am on a very important investigation! [A]",
                            "JULIAN: ??? [A]",
                            "INSPECTOR METHODICAL: Time is of the essence - this is a kidnapping case. [A]",
                            "JULIAN: Is it Frederika that is missing, by any chance Inspector? [A]",
                            "INSPECTOR METHODICAL: !!! [A]",
                            "INSPECTOR METHODICAL: What do you know about this case Mr Jones? [A]",
                            "JULIAN: Nothing really. [A]",
                            "INSPECTOR METHODICAL: You are sounding very fishy sir!  [A]",
                            "CHECKPOINT",
                            "JULIAN: So is your case... ",),
        "Hotel Reception": ("MUSIC:Reception", "Hotel Reception ..."),
        "Hotel Manager's Office":("MUSIC:Managers_office", "The Hotel Manager's Office ..."),
        "Behind the Hotel": ("MUSIC:Park", "Behind the hotel ..."),
        "Julian's Office": ("ADD:245,370,report",
                           "MUSIC:Detectives_office",
                           "Julian Jones' Office ... [A]",
                           "CHECKPOINT",
                           "JULIAN: Thank goodness the annoying Ms Demure has gone.")
    },


### Act 5
    {
        "Crime Scene": ("MUSIC:Stop", "The Crime Scene ..."),
        "Detention Centre": ("MUSIC:Cell", "The Detention Centre ..."),
        "Cambridge Street": ("MUSIC:Street", "Cambridge Street ..."),
        "Hotel Reception": ("MUSIC:Reception", "Hotel Reception ..."),
        "Hotel Manager's Office":("MUSIC:Managers_office", "The Hotel Manager's Office ..."),
        "Behind the Hotel": ("MUSIC:Park", "Behind the Hotel ..."),
        "Julian's Office": ("MUSIC:Tension",
                           "Julian Jones' Office ... [A]",
                           "TELETYPE:Off",
                           "So, if you think you have all the evidence you need, it is time for YOU to decide who was the murderer? ... [A]",
                           "Choose:\n[A] Captain Tina Tenable\n[B] Herr Produkt\n[C] Norah\n[D] Nobody committed the murder")
    }
)

#### clue text
clues = {
        "mobile": ">>> Ms Castle's mobile contains a recent text:'Hi, Tina, Can you come to the car pool at the Hotel. My car won't start. Need help!'\n",
        "photo": ">>> Norah's photo appears to show Captain Tenable covered in the victim's blood.\n",
        "report": ">>> The autopsy report says the victim appears to have died from carbon monoxide poisoning."
        }

#### end game text
answers = {
        "A": "Tina was Ms Castle's friend. She had come to help her. It was not the victim's blood on her coat. It was in fact grime and oil – Tina Tenable was NOT the murderer.\n\n[B] Herr Produkt\n[C] Norah\n[D] Nobody",
        "B": "Herr Produkt has an alibi. He was visiting another hotel he owned on the day of Ms Castle's death - Herr Produkt was NOT the murderer.\n\n[A] Captain Tina Tenable\n[C] Norah\n[D] Nobody",
        "C": "Norah was a possibility. She has no alibi and was known to be at the scene of Ms Castle's death. The evidence now points to another scenario though. - Norah was NOT the murderer.\n\n[A] Captain Tina Tenable\n[B] Herr Produkt\n[D] Nobody",
        "D": "CONGRATULATIONS - YOU HAVE SOLVED THE CASE!.\nJULIAN: Ms Castle's car had a fault in its exhaust causing fumes to go into the boot. While she was investigating she was overcome and fell in and subsequently died.\nCaptain Tenable tried to help but was also overcome with fumes but did not fit in the boot and was discovered later unconscious with memory loss, caused by carbon monoxide poisoning. [Quit]"
        }

####################################################################


from tkinter import *
import simpleAudio # imports: sys, platform, subprocess, time, threading

#### Set variables:
canvas_height=400
canvas_width=600
canvas_colour="black"
p1_x=canvas_width/2
p1_y=canvas_height

backpack=[]
current_coords = [0,1] # Crime scene
act = 0
script_counter = 0
checkpoint_counter = 0
teletype_noise = True

window = Tk()
window.title("The Mysterious Mystery")

#### import all images and assign to scenes
behind_the_hotel = PhotoImage(file="images/scene_behind_building.gif")
title = PhotoImage(file="images/title.gif")
crime_scene = PhotoImage(file="images/scene_crime_scene.gif")
crime_scene2 = PhotoImage(file="images/scene_crime_scene2.gif")
crime_scene_dark = PhotoImage(file="images/scene_crime_scene_dark.gif")
detention_centre = PhotoImage(file="images/scene_detention_centre.gif")
hotel_managers_office = PhotoImage(file="images/scene_hotel_office.gif")
hotel_reception = PhotoImage(file="images/scene_hotel_reception.gif")
julians_office = PhotoImage(file="images/scene_julians_office.gif")
cambridge_street = PhotoImage(file="images/scene_street.gif")

# load images for characters
Tina = PhotoImage(file="images/tina_near.gif")
Mike = PhotoImage(file="images/mike-near.gif")
Mike_head = PhotoImage(file="images/mike-head.gif")
Demi = PhotoImage(file="images/demi.gif")
Produkt = PhotoImage(file="images/produkt.gif")
Arnold = PhotoImage(file="images/arnold_near.gif")
Norah = PhotoImage(file="images/norah.gif")
Julian = PhotoImage(file="images/julian_near.gif")

# load images for clues
grey = PhotoImage(file="images/screen.gif") # for background to inventory
photo = PhotoImage(file="images/photo.gif")
photo_small = PhotoImage(file="images/photo_small.gif")
mobile = PhotoImage(file="images/mobile.gif")
mobile_small = PhotoImage(file="images/mobile_small.gif")
report_small = PhotoImage(file="images/report_small.gif")
report = PhotoImage(file="images/report.gif")

# Game interface images
bag_image = PhotoImage(file="images/backpack.gif")
grabber_image = PhotoImage(file="images/grab.gif")
search_image = PhotoImage(file="images/look.gif")
up_image = PhotoImage(file="images/up.gif")
left_image = PhotoImage(file="images/left.gif")
right_image = PhotoImage(file="images/right.gif")
down_image = PhotoImage(file="images/down.gif")
quit_image = PhotoImage(file="images/quit.gif")
a_image = PhotoImage(file="images/a-button.gif")
b_image = PhotoImage(file="images/b-button.gif")
c_image = PhotoImage(file="images/c-button.gif")
d_image = PhotoImage(file="images/d-button.gif")

# To enable characters, acts and items to be added from the script the image objects
# need to be mapped to strings. This is achieved with a dictionary:
image_dictionary = {
    "Tina":Tina,
    "Mike":Mike,
    "Mike_head":Mike_head,
    "Demi":Demi,
    "Produkt":Produkt,
    "Arnold":Arnold,
    "Norah":Norah,
    "Julian":Julian,
    "title":title,
    "behind_the_hotel":behind_the_hotel,
    "crime_scene":crime_scene,
    "crime_scene2":crime_scene2,
    "crime_scene_dark":crime_scene_dark,
    "detention_centre":detention_centre,
    "hotel_managers_office":hotel_managers_office,
    "hotel_reception":hotel_reception,
    "julians_office":julians_office,
    "cambridge_street":cambridge_street,
    "photo":photo_small,
    "photo_large":photo,
    "mobile":mobile_small,
    "mobile_large":mobile,
    "report":report_small,
    "report_large":report
    }

# Import Audio
Rain = 'audio/2520__rhumphries__rbh-rain-02.wav'
Cell = 'audio/16836__dobroide__20060311-elevator-cvp.wav'
Street = 'audio/168713__klankbeeld__briliant-guitarplay-02-close.wav'
Managers_office = 'audio/30191__erh__cello14-loop.wav'
Park = 'audio/32798__dobroide__20070325-piru-piru-piru.wav'
Detectives_office= 'audio/103557__greencouch__beach-waves-close.wav'
Tension = 'audio/149300__setuniman__worry-0k-30mi2-noncommercial.wav'
Reception = 'audio/61320__mansardian__news-signature-long.wav'
Grab_stuff = 'audio/150220__killkhan__reload-1-noncommercial.wav'
Tada = 'audio/177120__rdholder__2dogsound-tadaa1-3s-2013jan31-cc-by-30-us.wav'
Click = 'audio/click.wav'

# Create audio dictionary for same reason image one needed
audio_dictionary = {
    "Grab":Grab_stuff,
    "Rain":Rain,
    "Cell":Cell,
    "Street":Street,
    "Managers_office":Managers_office,
    "Park":Park,
    "Detectives_office":Detectives_office,
    "Tension":Tension,
    "Reception":Reception,
    "Tada":Tada
    }

# Build the Game Map:
scenes = (
    ({},{},{},{}),
    ({},{},{},{}),
    ({},{},{},{}),
    ({},{},{},{})
    )

# Load the map with scenes
scenes[0][1]["name"] = "Crime Scene"
scenes[0][1]["backdrop"] = "title"

scenes[1][1]["name"] = "Detention Centre"
scenes[1][1]["backdrop"] = "detention_centre"

scenes[2][1]["name"] = "Cambridge Street"
scenes[2][1]["backdrop"] = "cambridge_street"

scenes[2][2]["name"] = "Hotel Reception"
scenes[2][2]["backdrop"] = "hotel_reception"

scenes[3][2]["name"] = "Hotel Manager's Office"
scenes[3][2]["backdrop"] = "hotel_managers_office"

scenes[2][3]["name"] = "Behind the Hotel"
scenes[2][3]["backdrop"] = "behind_the_hotel"

scenes[2][0]["name"] = "Julian's Office"
scenes[2][0]["backdrop"] = "julians_office"



#### Functions:
def teletype(words):
    my_string=words
    story_box.delete("1.0", END)
    for char in words:       
        story_box.insert(END, char)
        if char != " " and teletype_noise and teletype_sfx:
            simpleAudio.startSound(Click, async=True, loop=False)
            simpleAudio.time.sleep(0.025)
        window.update_idletasks()
        simpleAudio.time.sleep(0.025)

def image_loader(label, x=0, y=400):
    img = image_dictionary[label]
    canvas.create_image(x, y, image=img, anchor=SW)

def stage_directions(script):
    global checkpoint_counter
    global teletype_noise
    global script_counter
    new_script = False
    scene = get_scene()
    scene_name = scene["name"]

    if script == "CHECKPOINT":
        checkpoint_counter = checkpoint_counter+1
        new_script = True
    elif script[:5] == "MUSIC":
        simpleAudio.stopSound()
        if background_music:
            if script[6:] != "Stop":
                wav = audio_dictionary[script[6:]]
                simpleAudio.startSound(wav, async=True, loop=True)
        new_script = True
    elif script[:5] == "ENTER":
        scene["character"] = script[6:]
        new_script = True
    elif script[:4] == "EXIT":
        del scene["character"]
        new_script = True
    elif script[:8] == "TELETYPE":
        if script[9:] == "Off":
            teletype_noise = False
        if script[9:] == "On":
            teletype_noise = True
        new_script = True
    elif script[:3] == "ADD":
        item = script[12:]
        if item not in backpack:
            scene["item"] = script[4:]
        new_script = True
    elif script[:7] == "REMOVE":
        del scene["item"]
        script_counter = script_counter+1
        new_script = True
    elif script[:8] == "BACKDROP":
        scene["backdrop"] = script[9:]
        new_script = True
    if new_script:
        script_counter = script_counter+1
        new_text = acts[act][scene_name][script_counter]
        return stage_directions(new_text)
    else:
        return script

def tell_story():
    global script_counter
    check_if_act_changed()
    scene = get_scene()
    scene_name = scene["name"]
    
    # get text
    text = acts[act][scene_name][script_counter]
    
    #process text with stage_directions()
    text = stage_directions(text)
    
    teletype(text)
    script_counter = script_counter+1
    update_scenery()

def click(button):
    global checkpoint_counter
    global script_counter

    #De-bugging info sent to Python Shell
    #print("Game progress:\nAct:", act, "Checkpoints:", checkpoint_counter, "script counter:", script_counter)
    
    if button == "a" and act == 5 and get_scene()["name"] == "Julian's Office" and script_counter > 4:
        story_box.delete("1.0", END)
        teletype(answers["A"])
        
    elif button == "a":
        scene = get_scene()
        scene_name = scene["name"]

        if script_counter < (len(acts[act][scene_name])):
            story_box.delete("1.0", END)
            text = acts[act][scene_name][script_counter]
            text = stage_directions(text)
            update_scenery()
            teletype(text)
            script_counter = script_counter+1
            check_if_act_changed()
        
    elif button == "b" and act == 5 and get_scene()["name"] == "Julian's Office":
        story_box.delete("1.0", END)
        teletype(answers["B"])
    elif button == "c" and act == 5 and get_scene()["name"] == "Julian's Office":
        story_box.delete("1.0", END)
        teletype(answers["C"])
    elif button == "d" and act == 5 and get_scene()["name"] == "Julian's Office":
        canvas.create_image(150, 400, image = Julian, anchor = S)
        canvas.create_image(320, 400, image = Tina, anchor = S)
        simpleAudio.stopSound()
        simpleAudio.startSound(Tada, async=True, loop=False)
        simpleAudio.startSound(Street, async=True, loop=False)
        story_box.delete("1.0", END)
        teletype(answers["D"])
        canvas.create_image(500, 440, image = Mike, anchor = S)
    else:
        pass

def check_if_act_changed():
    global act
    global checkpoint_counter
    global scene_counter
    if checkpoint_counter > 2:
        act = act+1
        checkpoint_counter = 0
        scene_counter = 0
        # remove all characters from map
        for x in range(0,4):
            for y in range(0,4):
                if scenes[x][y]:
                    if "character" in scenes[x][y]:
                        del scenes[x][y]["character"]
        # leave items to be found later if not found in this act
                        
def update_scenery():
    scene = get_scene()
    scene_name = scene["name"]
    image_loader(scene["backdrop"])

    if "item" in get_scene():
        label = scene["item"][8:]
        x = scene["item"][:3]
        y = scene["item"][4:7]
        image_loader(label, x, y)

    if "character" in get_scene():
        label = scene["character"][8:]
        x = scene["character"][:3]
        y = scene["character"][4:7]
        image_loader(label, x, y)

        # debugging info sent to Python shell
        # print("Character coords: ", label, ",", x,",",y)

def get_scene():
    return scenes[current_coords[0]][current_coords[1]]

def look():
    story_box.delete("1.0", END)
    if "item" in get_scene():
        msg = "You can see a " + get_scene()["item"][8:]
    else:
        msg = "Nothing to see here."
    story_box.insert(END, msg)

def grab():
    story_box.delete("1.0", END)
    if "item" in get_scene():
        backpack.append(get_scene()["item"][8:])
        msg = "You have collected the " + get_scene()["item"][8:] + "!!!"
        simpleAudio.startSound(Grab_stuff, async=True, loop=False)
        del get_scene()["item"]
        update_scenery()
    else:
        msg = "Nothing to grab here."
    story_box.insert(END, msg)

def inventory():
    # This button toggles the viewability of the inventory (if there is any)
    # First we check to see if inventory is already showing and if it is we
    # close it by calling tell_story() to continue with the scene.
    if story_box.get("1.1") == ">":
        global script_counter
        script_counter = script_counter-1
        tell_story()
    else:
        story_box.delete("1.0", END)
        if len(backpack) != 0:
            canvas.create_image(310, 200, image=grey, anchor=CENTER)
            if "mobile" in backpack:
                msg = clues["mobile"]
                story_box.insert(END, msg)
                image_loader("mobile_large", 80, 250)
            if "photo" in backpack:
                msg = clues["photo"]
                story_box.insert(END, msg)
                image_loader("photo_large", 350, 200)
            if "report" in backpack:
                msg = clues["report"]
                story_box.insert(END, msg)
                image_loader("report_large", 210, 320)
        else:
            msg = "You have not collected anything yet."
            story_box.insert(END, msg)

def is_off_map():
    x = current_coords[1]
    y = current_coords[0]
    if (x < 0) or (x > 3) or (y < 0) or (y > 3):
        return True
    else:
        return False
    
def is_no_scene():
    if get_scene():
        return False
    else:
        return True

def quit_game():
    simpleAudio.stopSound()
    window.withdraw()
    window.quit()

def move(y,x):
    global current_coords
    global script_counter
    current_coords[0] = current_coords[0] + x
    current_coords[1] = current_coords[1] + y

    story_box.delete("1.0", END)

    if is_off_map() or is_no_scene():
        current_coords[0] = current_coords[0] - x
        current_coords[1] = current_coords[1] - y
        teletype("You have bumped into a wall.")
    else:
        script_counter = 0
        tell_story()

def move_north():
    move(0,-1)

def move_south():
    move(0,1)

def move_east():
    move(1,0)

def move_west():
    move(-1,0)

def a_press():
    click("a")

def b_press():
    click("b")

def c_press():
    click("c")

def d_press():
    click("d")

#### main:


# create frames
topframe = Frame(window)
topframe.grid(row=0, column=0, columnspan=3, sticky=N)

bottom_left = Frame(window)
bottom_left.grid(row=1, column=0, sticky=N)

bottom_centre = Frame(window)
bottom_centre.grid(row=1, column=1, sticky=N)

bottom_right = Frame(window)
bottom_right.grid(row=1, column=2, sticky=N)

# setup canvas
canvas = Canvas(topframe, bg=canvas_colour, height=canvas_height-3, width=canvas_width-3)
canvas.grid(row=0, column=0)

# build story area
story_box = Text(topframe, height=7, width=80, wrap=WORD)
story_box.grid(row=1, column=0)

# build game buttons
Button(bottom_centre, image=search_image, command=look).grid(row=0, column=0)
Button(bottom_centre, image=bag_image, command=inventory).grid(row=0, column=1)
Button(bottom_centre, image=grabber_image, command=grab).grid(row=0, column=2)
Button(bottom_centre, image=quit_image, command=quit_game).grid(row=0, column=3)

# build bottom right set of buttons
Button(bottom_right, image=up_image, command=move_north).grid(row=0, column=1)
Button(bottom_right, image=left_image, command=move_west).grid(row=1, column=0)
Button(bottom_right, image=right_image, command=move_east).grid(row=1, column=2)
Button(bottom_right, image=down_image, command=move_south).grid(row=2, column=1)

# build bottom left set of buttons
Button(bottom_left, image=a_image, command=a_press).grid(row=0, column=2)
Button(bottom_left, image=b_image, command=b_press).grid(row=1, column=1)
Button(bottom_left, image=c_image, command=c_press).grid(row=1, column=3)
Button(bottom_left, image=d_image, command=d_press).grid(row=2, column=2)


#### main:
# start game
tell_story()

# Ensure music stops when the game is closed
window.protocol("WM_DELETE_WINDOW", quit_game)

# start tkinter's main loop
window.mainloop()
