# rpg-gui.py

from tkinter import *

#### Set variables:
canvas_height=400
canvas_width=600
paper_colour = "linen"
text_box_width = 80
text_box_height = 7

backpack = []
act = 0
current_location = [0,1]

# Build the Map
scenes = (
    ({},{},{}),
    ({},{},{}),
    ({},{},{})
    )

# Load the map with scenes
scenes[0][1]["name"] = "The Car"
scenes[0][1]["item"] = "mobile phone"

scenes[1][1]["name"] = "Yard"

scenes[1][2]["name"] = "Office"
scenes[1][2]["item"] = "sim card"

# Create stories for each scene
acts = (
    {
        "The Car": "You have left your phone somewhere but cannot find it.\nHow annoying :-(",
        "Yard": "It's a nice day. Now where is that phone?",
        "Office": "This is where Mum works.\nWe call it Mum's Office but she refers to it as 'My Sanctuary'."
    },
    {
        "The Car": "Your phone does not appear to work here.",
        "Yard": "Would I really have left my phone outside??",
        "Office": "Your phone does not appear to work here."
    },
    {
        "The Car": "Grr! My mobile gets no reception here either!",
        "Yard": "Finally, you have a working phone and a decent signal!\nRing ring...\nRing ring...\n\nPress 'Quit' to finish.",
        "Office": "You have a phone with a sim card but still cannot use it as there is no reception."
    }
)

# Functions
def tell_story():
    update_act()
    scene_name = get_scene()["name"]
    print("\nYou are in the " + scene_name + "\n")
    print(acts[act][scene_name])

def update_act():
    global act
    if("mobile phone" in backpack) and ("sim card" in backpack):
        act = 2
    elif "mobile phone" in backpack:
        act = 1
    return act

def get_scene():
    return scenes[current_location[0]][current_location[1]]

def look():
    if "item" in get_scene():
        print("You can see a " + get_scene()["item"])
    else:
        print("Nothing to see here.")

def grab():
    if "item" in get_scene():
        backpack.append(get_scene()["item"])
        print("You now have the " + get_scene()["item"])
        del get_scene()["item"]
    else:
        print("Nothing to grab here.")

def inventory():
    if len(backpack) != 0:
        print(backpack)
    else:
        print("You have not collected anything yet.")

def quit_game():
    window.withdraw()
    window.quit()

def move_north():
    current_location[0] = current_location[0]-1
    if is_off_map() or is_no_scene():
        current_location[0] = current_location[0]+1
        print("You have bumped into a wall.")

def move_south():
    current_location[0] = current_location[0]+1
    if is_off_map() or is_no_scene():
        current_location[0] = current_location[0]-1
        print("You have bumped into a wall.")

def move_east():
    current_location[1] = current_location[1]+1
    if is_off_map() or is_no_scene():
        current_location[1] = current_location[1]-1
        print("You have bumped into a wall.")

def move_west():
    current_location[1] = current_location[1]-1
    if is_off_map() or is_no_scene():
        current_location[1] = current_location[1]+1
        print("You have bumped into a wall.")

def is_no_scene():
    if get_scene():
        return False
    else:
        return True

def is_off_map():
    x = current_location[1]
    y = current_location[0]
    if (x < 0) or (x > 2) or (y < 0) or (y > 2):
        return True
    else:
        return False

#### Main:
window = Tk()
window.title("MyAdventure")

# Load images for scenes
car = PhotoImage(file="images/scene_car.gif")

# Load images for move buttons
up = PhotoImage(file="images/up.gif")
left = PhotoImage(file="images/left.gif")
right = PhotoImage(file="images/right.gif")
down = PhotoImage(file="images/down.gif")

# Load images for bottom buttons
bag = PhotoImage(file="images/backpack.gif")
grabber = PhotoImage(file="images/grab.gif")
search = PhotoImage(file="images/look.gif")
quit_image = PhotoImage(file="images/quit.gif")

# Create top frame
topframe = Frame(window)
topframe.grid(row=0, column=0, columnspan=3, sticky=N)

# Create bottom_left frame
bottom_left = Frame(window)
bottom_left.grid(row=1, column=0, sticky=NW)

# Create bottom_centre frame
bottom_centre = Frame(window)
bottom_centre.grid(row=1, column=1, sticky=N)

# Create bottom_right frame
bottom_right = Frame(window)
bottom_right.grid(row=1, column=2, sticky=NE)

# Setup canvas
canvas = Canvas(topframe, height=canvas_height, width=canvas_width, highlightthickness=0)
canvas.grid(row=0, column=0)

# Build story area
story_box = Text(topframe, height=text_box_height, width=text_box_width, background=paper_colour)
story_box.grid(row=1, column=0)

# Direction buttons
Button(bottom_right, image=up, command=move_north).grid(row=0, column=2)
Button(bottom_right, image=left, command=move_west).grid(row=1, column=1)
Button(bottom_right, image=right, command=move_east).grid(row=1, column=3)
Button(bottom_right, image=down, command=move_south).grid(row=2, column=2)

# Other buttons
Button(bottom_centre, image=search, command=look).grid(row=0, column=0)
Button(bottom_centre, image=bag, command=inventory).grid(row=0, column=1)
Button(bottom_centre, image=grabber, command=grab).grid(row=0, column=2)
Button(bottom_left, image=quit_image, command=quit_game).grid()

# Start game
canvas.create_image(0, 0, image=car, anchor=NW)
story_box.insert(END, "text goes here") 

# Start tkinter's main loop
window.mainloop()
